<?php
	// An abstraction of a news item.
	class BVCNewsItem {
		public $newsID;
		public $newsDate;
		public $newsTitle;
		public $newsText;
		public $newsSticky;
	}

	// An abstraction of a track.
	class BVCTrackItem {
		public $trackID;
		public $trackName;
		public $trackURL;
		public $artistName;
		public $artistEmail;
		public $entryDate;
		public $entryVotable;
		public $entryConfirmed;
		public $inspirationID;
		// The following are not always used.
		public $votesFirst;
		public $votesSecond;
		public $votesThird;
		public $votesTotalPoints;
	}

	// An abstraction of a competition round.
	class BVCRound {
		public $tracks = array();
		public $inspirationID;
		public $roundDate;

		public function sortTracks() {
			usort($this->tracks, array($this, "comparePoints"));
		}

		private function comparePoints($a, $b) {
			if ($a->votesTotalPoints == $b->votesTotalPoints) {
				// If they're equal, decide the tie-break by the number of votes for first place.
				if ($a->votesFirst > $b->votesFirst) {
					return 1;
				} elseif ($a->votesFirst < $b->votesFirst) {
					return -1;
				} else {
					// If they're still equal, go to second place.
					if ($a->votesSecond > $b->votesSecond) {
						return 1;
					} elseif ($a->votesSecond < $b->votesSecond) {
						return -1;
					} else {
						// If they're still equal, go to third place.
						if ($a->votesThird > $b->votesThird) {
							return 1;
						} elseif ($a->votesThird < $b->votesThird) {
							return -1;
						} else {
							return 0; // Nothing more we can do.
						}
					}
				}
			}
			return ($a->votesTotalPoints > $b->votesTotalPoints) ? -1 : 1;
		}
	}


	// An abstraction of an RSS 2.0 item.
	class rssItem {
		public $title;
		public $link;
		public $description;
		public $publicationDate;
	}

	// An abstraction of an RSS 2.0 feed.
	class rssFeed {
		public $rssItems = array();
		public $rssTitle;
		public $rssURL;
		public $rssDescription;
		public $rssLanguage = "en-uk";
		private $rssGenerator = "The Barryvan Compo Manager";

		public function getFeed() {
			if (empty($this->rssItems)) {
				return "";
			}
			
			// First, we want the array nicely sorted.
			usort($this->rssItems, array($this, "compareDates"));
			$feedString  = "<?xml version=\"1.0\"?>\n";
			$feedString .= "<rss version=\"2.0\">\n";
			$feedString .= "\t<channel>\n";
			$feedString .= "\t\t<title>$this->rssTitle</title>\n";
			$feedString .= "\t\t<link>$this->rssURL</link>\n";
			$feedString .= "\t\t<description>$this->rssDescription</description>\n";
			$feedString .= "\t\t<language>$this->rssLanguage</language>\n";
			$feedString .= "\t\t<lastBuildDate>" . $this->getNewestDate($this->rssItems) . "</lastBuildDate>\n";
			$feedString .= "\t\t<pubDate>" . date("r") . "</pubDate>\n";
			$feedString .= "\t\t<generator>$this->rssGenerator</generator>\n";
			$feedString .= "\t\t<docs>http://blogs.law.harvard.edu/tech/rss</docs>\n";

			foreach ($this->rssItems as $item) {
				$feedString .= "\t\t<item>\n";
				$feedString .= "\t\t\t<title>" . $item->title . "</title>\n";
				$feedString .= "\t\t\t<link>" . $item->link . "</link>\n";
				$feedString .= "\t\t\t<description>" . $item->description . "</description>\n";
				$feedString .= "\t\t\t<pubDate>" . date("r", $item->publicationDate) . "</pubDate>\n";
				$feedString .= "\t\t</item>\n";
			}

			$feedString .= "\t</channel>\n";
			$feedString .= "</rss>\n";

			return $feedString;
		}

		private function getNewestDate() {
			// Pre: The array is sorted by newest date first.
			return date("r", $this->rssItems[0]->publicationDate);
		}

		private static function compareDates($a, $b) {
			if ($a->publicationDate == $b->publicationDate) {
				return 0;
			}
			return ($a->publicationDate > $b->publicationDate) ? -1 : 1;
		}
	}
		

	
	
	// Manages an inspiration-based compo, like the barryvan compo.
	class InspirationCompo {
		// TODO
	}
?>
